#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define BACKLOG 5

#define __DEBUG 


int prepare_for_connections(int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  int yes=1;

  /* prparation de la chaussette du serveur */
  /* on prcise le numro de port, l'adresse IP */
  srv_socket_adr.sin_family = AF_INET;         
  srv_socket_adr.sin_port = htons(portnum);     
  srv_socket_adr.sin_addr.s_addr = INADDR_ANY; /* mon adresse IP */
  memset(&(srv_socket_adr.sin_zero), '\0', 8); 

  /* obtention d'un descripteur */
  if ( (srv_socket=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("server:socket");
      return -3;

    }
  if (setsockopt(srv_socket,
		 SOL_SOCKET,
		 SO_REUSEADDR, &yes, sizeof(int)) == -1) {
    perror("setsockopt");
    exit(1);
  }

  /* on associe la chaussette  l'adresse IP/N port du serveur */
  if ( bind (srv_socket, 
	     (struct sockaddr*)&srv_socket_adr, 
	     sizeof(struct sockaddr)) == -1 )
    {
      perror("server:bind");
      return -4;
    }

  /* on coute */
  if ( listen(srv_socket,BACKLOG) == -1 )
    {
      perror("server:listen");
      return -5;
    }

  return srv_socket;
}

int connect_to_server_by_ip(char* ipserveur, int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  struct in_addr ip_adr;

  inet_aton(ipserveur,&ip_adr);

  /* prparation de la chaussette */
  /* on prcise l'adresse du serveur et le numro de port */
  memset((void*)&srv_socket_adr,0,sizeof(srv_socket_adr));

  srv_socket_adr.sin_addr=ip_adr;
  srv_socket_adr.sin_family=AF_INET;
  srv_socket_adr.sin_port=htons(portnum);

  /* obtention du descripteur */
  if ( (srv_socket=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("socket");
      return -1;
    }

  /* connexion au serveur */
  if ( connect(srv_socket, 
	       (struct sockaddr*)&srv_socket_adr, 
	       sizeof(struct sockaddr) ) == -1 )
    {
      perror("connect");
      return -2;
    }
#ifdef __DEBUG
  printf("connexion  %s\n",ipserveur);
#endif
  return srv_socket;
}

int connect_to_server_by_name(char* nomserveur, int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  struct hostent *host;

  /* adresse du serveur dont on connat le nom */
  if ( (host=gethostbyname(nomserveur)) == NULL )
    {
      perror("gethostbyname");
      return -1;
    }

  /* obtention du descripteur */
  if ( (srv_socket=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("socket");
      return -2;
    }

  /* prparation de la chaussette */
  /* on prcise l'adresse du serveur et le numro de port */
  srv_socket_adr.sin_family=AF_INET;
  srv_socket_adr.sin_port=htons(portnum);
  srv_socket_adr.sin_addr=*((struct in_addr *)host->h_addr);
  memset(&srv_socket_adr.sin_zero,'\0',8);


  /* connexion au serveur */
  if ( connect(srv_socket, 
	       (struct sockaddr*)&srv_socket_adr, 
	       sizeof(struct sockaddr) ) == -1 )
    {
      perror("connect");
      return -3;
    }
  return srv_socket;
}
extern int errno;

int get_connexion(int srv_sd,struct sockaddr_in *cltadr)
{
  socklen_t fromlen = sizeof(struct sockaddr_in);
  int clt_sd;

  /* on rcupre l'adresse du client qui s'est connect */
  if (  (clt_sd=accept(srv_sd,(struct sockaddr*)cltadr,&fromlen)) == -1 )
    {
      if (errno!=EINTR)
	perror("accept");
      return -1;
    }
#ifdef __DEBUG
  printf("connexion de : %s\n",inet_ntoa(cltadr->sin_addr));
#endif
  return clt_sd;
}

int send_message(int sd, char* msg)
{
  int sentcars;

  /* envoi du message */
  if ( (sentcars=send ( sd, msg, strlen(msg)+1,0 )) == -1 )
    {
      return -1;
    }
  return sentcars;
}
