#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>

#include "demon.h"

void log_message(const char* filename, 
		 const char* action, const char* message)
{
  FILE *logfile;
  char str_date[150];
  time_t date=time(NULL);
  struct tm *tm_date=localtime(&date);
  
  logfile=fopen(filename,"a");
  if(!logfile) 
    return;

  strftime(str_date,150,"%F %T",tm_date);
  if (message)
    fprintf(logfile,"[%s] (%d) %s %s\n",str_date,getpid(),action,message);
  else
    fprintf(logfile,"[%s] (%d) %s\n",str_date,getpid(),action);    
  fclose(logfile);
}



void daemonize(int uid, const char* lckfile, const char* logfile, char* rundir)
{
  int i;

  if(getppid()==1) return; /* already a daemon */

  i=fork();
  if (i<0) exit(1); /* fork error */
  if (i>0) exit(0); /* parent exits */


  if (setuid(uid)==-1)
    {
      perror("setuid");
      exit(1);
    }

  /* child (daemon) continues */
  if (setsid()==-1) /* obtain a new process group */
    {
      perror("setsid");
      exit(1);
    }

  for (i=getdtablesize();i>=0;--i) close(i); /* close all descriptors */

  i=open("/dev/null",O_RDWR); dup(i); dup(i); /* handle standart I/O */


  umask(027); /* set newly created file permissions */

  chdir(rundir); /* change running directory */

  store_pid_and_lock(lckfile,logfile);

}

int store_pid_and_lock(const char* nomfichier, const char* logfile)
{
  int  lfp;
  char str[10];

  lfp=open(nomfichier,O_RDWR|O_CREAT|O_EXCL,0640);
  if (lfp<0) {
    log_message(logfile,
		"Ouverture fichier lck : ",
		strerror(errno));
    exit(1); 
  }
  if (lockf(lfp,F_TLOCK,0)<0) 
    {
      log_message(logfile,"[store_pid_and_lock] pas pu verrouiller",strerror(errno));
      exit(0); 
    }
  sprintf(str,"%d\n",getpid());
  write(lfp,str,strlen(str));
  return 0;
}

int read_pid(const char* nomfichier, const char* logfile)
{
  FILE* f=fopen(nomfichier,"r");
  int pid;

  if (f==NULL)
    {
      log_message(logfile,"[read_pid] Ouverture du fichier lock",strerror(errno));
      return -1;
    }
  if (fscanf(f,"%d",&pid)==1)
    return pid;
  
  log_message(logfile,"[read_pid] pas de pid dans le fichier lock",NULL);
  return -1;
}
