#include "lock.h"
#include <string.h>
#include <errno.h>

static  void unlockscreen(Display *dpy, struct st_lock *lock) 
{
  if (dpy == NULL || lock == NULL)
    return;
  
  XUngrabPointer(dpy, CurrentTime);
  XFreePixmap(dpy, lock->pmap);
  XDestroyWindow(dpy, lock->w);
  
  free(lock);
}

static struct st_lock * lockscreen(Display *dpy, int screen)
{
  char curs[] = {0, 0, 0, 0, 0, 0, 0, 0};
  unsigned int len;
  struct st_lock *lock;
  Bool running = True;
  XColor black, dummy;
  XSetWindowAttributes wa;
  Cursor invisible;
  
  if (dpy == NULL || screen < 0)
    return NULL;
  
  lock = malloc(sizeof(struct st_lock));
  if (lock == NULL)
    return NULL;
  
  lock->screen = screen;
  
  lock->root = RootWindow(dpy, lock->screen);
  
  /* init */
  wa.override_redirect = 1;
  wa.background_pixel = BlackPixel(dpy, lock->screen);
  lock->w = XCreateWindow(dpy, lock->root, 0, 0, 
			  DisplayWidth(dpy, lock->screen), 
			  DisplayHeight(dpy, lock->screen),
			  0, DefaultDepth(dpy, lock->screen), 
			  CopyFromParent,
			  DefaultVisual(dpy, lock->screen), 
			  CWOverrideRedirect | CWBackPixel, &wa);

  XAllocNamedColor(dpy, 
		   DefaultColormap(dpy, lock->screen), 
		   "black", &black, &dummy);

  lock->pmap = XCreateBitmapFromData(dpy, lock->w, curs, 8, 8);
  invisible  = XCreatePixmapCursor(dpy, 
				   lock->pmap, lock->pmap, 
				   &black, &black, 0, 0);
  XDefineCursor(dpy, lock->w, invisible);

  XMapRaised(dpy, lock->w);

  for(len = 1000; len; len--) 
    {
      if(XGrabPointer(dpy, lock->root, False, 
		      ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
		      GrabModeAsync, GrabModeAsync, 
		      None, invisible, CurrentTime) == GrabSuccess)
	break;
      usleep(1000);
    }
  if((running = running && (len > 0))) 
    {
      for(len = 1000; len; len--) 
	{
	  if(XGrabKeyboard(dpy, lock->root, True, 
			   GrabModeAsync, GrabModeAsync, CurrentTime)
	     == GrabSuccess)
	    break;
	  usleep(1000);
	}
      running = (len > 0);
    }
  
  if (!running) {
    unlockscreen(dpy, lock);
    lock = NULL;
  }
  
  return lock;
}



int lock_display(Display *dpy, struct st_lock ***locks)
{
  int screen,nscreens;

  /* Get the number of screens in display "dpy" and blank them all. */
  nscreens = ScreenCount(dpy);
  *locks = malloc(sizeof(struct st_lock *) * nscreens);
  if (locks == NULL)
    {
      fprintf(stderr,"malloc: %s", strerror(errno));
      exit(EXIT_FAILURE);
    }
  for (screen = 0; screen < nscreens; screen++)
    (*locks)[screen] = lockscreen(dpy, screen);
  
  XSync(dpy, False);
  return nscreens;
}

int unlock_display(Display *dpy, struct st_lock **locks,int nscreens)
{
  int screen;

  for (screen = 0; screen < nscreens; screen++)
    unlockscreen(dpy, locks[screen]);
  
  free(locks);
  return 0;
}
