#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <regex.h>


#include "chaussette.h"
#include "demon.h"
#include "lock.h"

struct sigaction action;

#define JECOUTED_USER 0
#define JECOUTED_PORT 6000

int srv_socket;
int port;
char* rundir;
char* lckfile;
char* logfile;
char* blkfile;

Display *dpy;
  

/*
  STOOOOP : handler de signaux

  sig : signal reu

  CHLD
    attendre la fin du fils
    
*/

void stoooop(int sig)
{
  switch(sig)
    {
    case SIGINT:
    case SIGTERM: /* arrt du daemon */
      unlink(lckfile);
      log_message(logfile,"jecouted stopped",NULL);
      exit(1);
      break;
    case SIGUSR1:
      unlink(blkfile);
      log_message(logfile,"Unlock screen",NULL);
      exit(1);
      break;
      /* lorsqu'un process se termine il envoie SIGCHLD  son pre */
    case SIGCHLD:
      wait(NULL);
      break;
    }
}

/*
  DMARRAGE DU JECOUTED

  1. on rcupre le numro de port 
  2. on rcupre le nom du rpertoire de travail
  3. on rcupre le nom des fichiers lck/log du daemon
  4. on installe le handler de signaux (KILL/TERM/INT/CHLD/USR1)
  5. on attend les messages :
     a. JECOUTE?
*/
int main(int argc, char** argv)
{
  int pidblack;
  int clt_socket;
  struct sockaddr_in clt_socket_adr;
  char buf[512];
  int code;
  int nscreens;
  struct st_lock **locks;

  port=JECOUTED_PORT;

  rundir="/var/run";
  lckfile="/var/run/jecoute.pid";
  logfile="/var/log/jecoute.log";
  blkfile="/var/run/jecoute.blk";

  daemonize(JECOUTED_USER,lckfile,logfile,rundir);

  action.sa_handler=stoooop;
  sigaction(SIGTERM,&action,NULL);
  sigaction(SIGINT,&action,NULL);
  sigaction(SIGCHLD,&action,NULL);
  sigaction(SIGUSR1,&action,NULL);

  log_message(logfile,"jecouted started",NULL);

  if ( (srv_socket=prepare_for_connections(port))<0 )
    {
      sprintf(buf,"Echec de la mise en place de la connexion (%d)",srv_socket);
      log_message(logfile,buf, strerror(errno));
      unlink(lckfile);
      exit(2);
    }

  /* boucle d'attente */
  for(;;)
    {
      if ( (clt_socket=get_connexion(srv_socket,&clt_socket_adr))!=-1)
	{
	  if (!fork()) /* processus fils */
	    {
	      /* on ferme la socket du serveur */
	      if (close(srv_socket))
		{
		  log_message(logfile,"close(srv_socket)",strerror(errno));
		}
	      
	      /* on reoit les infos de ce client */
	      if ( (recv(clt_socket,buf,sizeof(buf),0)) == -1 )
		{
		  log_message(logfile,"recv",strerror(errno));
		}
	      if (sscanf(buf,"LACHEZ %d",&code)==1)
		{
		    if(!(dpy = XOpenDisplay(NULL)))
		      log_message(logfile,"XOpenDisplay",strerror(errno));
		    else
		      {
			nscreens=lock_display(dpy,&locks);
			if ( store_pid_and_lock(blkfile,logfile)!=0 )
			  XCloseDisplay(dpy);
			else
			  {
			    log_message(logfile,"LOCK",NULL);
			    sleep(10000);
			  }
		      }
		}
	      else if (sscanf(buf,"VOUS POUVEZ %d",&code)==1)
		{
		  log_message(logfile,"UNLOCK",NULL);
		  pidblack=read_pid(blkfile,logfile);
		  if (pidblack>0)
		    kill(pidblack,SIGUSR1);
		  else
		    log_message(logfile,"SCREEN NOT LOCKED",NULL);
		}
	      else
		{
		  log_message(logfile, buf, inet_ntoa(clt_socket_adr.sin_addr));
		}
	      
	      /* on ferme la chaussette du client */
	      if (close(clt_socket))
		{
		  log_message(logfile,"close(clt_socket)",strerror(errno));
		}
	      exit(0);
	    }
	  /* on ferme la chaussette du client */
	  if (close(clt_socket))
	    {
	      log_message(logfile,"close(clt_socket)",strerror(errno));
	    }
	}
    }
  return 0;
  
}
